#Este programa calcula el CL y CD de un perfil doble cuna supersonico usando teoria linearizada
#Es muy sencillo de usar, se debe elegir con las barras delsizantes el angulo de ataque, espesor relativo y numero de mach deseados
#Los resultados se entregan en la parte inferior de la ventana, donde se puede ver el CL y CD del perfil (este ultimo incluye CD inducido debido a la fuerza aerodinamica y CD de forma producido por el espesor relativo, pero no contempla la friccion)
#Bajo el CL y CD se ve la sustentacion y resistencia, como funcion del area del ala, es decir, por metro cuadrado.
#Para resaltar las limitaciones de la teoria, el perfil puede toma de tres colores de relleno, negro, amarillo o rojo
#El color negro indica que ningun quiebre supera los 5 grados, por lo que la teoria es valida y los resultados precisos. 
#Si el perfil se pinta amarillo, significa que algun quiebre es mayor a 5 grados, y los resultados pierden precision.
#Si el perfil se pinta rojo, significa que hay un quiebre de mas de 10 grados, y los resultados ya no son precisos, ya que la teoria linearizada no es valida en esas condiciones.
#Las lineas de mach estan dibujadas con su angulo real (segun teoria lineal, no choque-expansion), por lo que se puede ver como se mueven con el numero de mach.
#Las lineas rojas representan ondas de compresion, puede notarse que estas ondas aumentan de espesor a medida que el quiebre que las produce se incrementa, representando su aumento de intensidad.
#Las lineas azules representan ondas de expansion, estas son siempre infinitesimales, por lo que no varian de espesor
#Al cliquear el boton de calcula, el programa crea tres graficos, uno de CL y CD vs alfa (usando como valor de mach el seleccionado en la barra deslizante), 
#Otro de CL y CD vs mach (usando el angulo de ataque elegido)
#Finalmente uno de CL vs CD (comparados en funcion de alfa, para el mach elegido)
#
#
 



import Tkinter as Tkinter
import math
import matplotlib.pyplot as plt

#parametros ventana principal
w_max=800.0     # ancho de ventana
h_max=400.0     # alto de ventana

##Variables inicializadas globalmente
c=400. ##cuerda en pixeles
esp=1. #espesor relativo inicial
M=1.2 #numero de mach inicial
alfa=0. #angulo de ataque inicial
beta=[0,0,0,0,0,0] #lista de angulos de cada linea de mach, en cada uno de los 6 quiebres
delta=math.atan((0.5*c*esp/100.)/(c/2))+alfa #angulo de quiebre del primer panel del extrados, es tita-alfa (pero alfa positivo es negativo en este sistema de referencia)


i=0 #i de contadores
alfa2=-5./57.295 #angulo de ataque inicial para calcular los graficos, en radianes
M2=1.2 #mach inicial para hacer los graficos


##calcular calcula las listas de cl/cd vs alfa y cl/cd vs mach, tambien grafica, se inicia con el boton de calcular
def calcular(tita):
    clalfa=[] #lista de cl vs alfa
    clmach=[] #lista de cl vs mach
    cdalfa=[] #lista de cd vs alfa
    cdmach=[] #lista de cd vs mach
    alfaplot=[] #lista con los alfa usados en el calculo de las listas cd/cl vs alfa, para usar como eje x en el grafico
    machplot=[] #idem anterior pero con mach
    i=0 #se inicializa i cada vez que se llama a calcular
    alfa2=-5./57.295 #idem anterior
    M2=1.2 #idem anterior
    
	#tomo solo 100 valores, alcanzan para que el grafico sea suave, alfa va de -5 a 5 grados y mach de 1.2 a 4
    for i in range(100):
        clalfa.append(((4*(alfa2))/((M**2-1)**0.5))) #calculo cl con la formula del shapiro para pequenas perturbaciones. Barre alfa de -5 a 5 con el mach seleccionado en el slider
        clmach.append(((4*(-alfa))/((M2**2-1)**0.5))) #calculo cl con la formula del shapiro para pequenas perturbaciones. Barre mach de 1.2 a 4 con el alfa seleccionado en el slider
		        
        cdalfa.append(((4*(-alfa2)**2)/((M**2-1)**0.5)+(4*tita**2)/((2**2-1)**0.5))) #idem anterior pero para cd
        cdmach.append(((4*(-alfa)**2)/((M2**2-1)**0.5)+(4*tita**2)/((M2**2-1)**0.5)))#idem
        
        alfaplot.append(alfa2*57.295) #agrego a la lista de alfas(para eje x del grafico) el alfa 2 con el que calcule cl y cd, pasado de rad a grados
        machplot.append(M2)#idem anterior pero con el mach
        M2=M2+0.028#incremento necesario para barrer de M=1.2 a M=4 en 100 pasos
        alfa2=alfa2+(0.1/57.295)#incremento necesario para barrer de -5 a 5 grados en 100 pasos, pasado a radianes
    
    
    
    
    ##grafico cl y cd
    fig = plt.figure() #crea el grafico
    plt.rcParams["figure.figsize"] = [10,10] #amplia la figura a 10x10 pulgadas
    alfacl = fig.add_subplot(311) #crea el subplot de cl vs alfa
    alfacl.plot(alfaplot, clalfa, color='blue', linewidth=3) #valores de la linea de cl vs alfa
    alfacl.grid(True,linestyle='--')    #tipo de linea de la grid
    plt.xlabel('Alfa',fontsize=12) #nombre del eje x
    plt.ylabel('CL - CD',fontsize=12) #nombre del eje y
    plt.title('CL - CD vs Alfa',fontsize=16) #nombre del grafico
    
    
    
    machcl = fig.add_subplot(312) #idem anterior pero para otro subplot
    machcl.plot(machplot, clmach, color='blue', linewidth=3)
    machcl.grid(True,linestyle='--')
    plt.xlabel('Mach',fontsize=12)
    plt.ylabel('CL - CD',fontsize=12)
    plt.title('CL - CD vs Mach',fontsize=16)
    
    
    
    alfacd = fig.add_subplot(311) #idem anterior, al usar la misma lista de eje x que el primer grafico, los superpone
    alfacd.plot(alfaplot, cdalfa, color='red', linewidth=3)
    alfacd.grid(True,linestyle='--')
    plt.legend(("CL","CD"))
    plt.text(4., clalfa[99]*1.2, 'Mach = '+str(M)) #muestra el mach usado en la parte superior derecha del grafico
        
    
    machcd = fig.add_subplot(312)#idem anteriores, para cd vs mach
    machcd.plot(machplot, cdmach, color='red', linewidth=3) 
    plt.legend(("CL","CD"))
    if alfa==0: #muestra el alfa usado en la esquina superior derecha, como el cl puede ser cero, para todo mach, no puedo usar el mismo metodo que antes y tengo que ver para cada caso, de todas formas da problemas para alfas negativos 
        plt.text(3.9, cdmach[0]*1.1, r'$\alpha = $'+str(round(-alfa*57.295,0)))
    elif -alfa>0:
        plt.text(3.9, clmach[0]*1.1, r'$\alpha = $'+str(round(-alfa*57.295,0)))
    else:
        plt.text(3.9, cdmach[0]*1.8, r'$\alpha = $'+str(round(-alfa*57.295,0)))
    
    
    clcd = fig.add_subplot(313) #grafico de cl vs cd, uso cl como eje y y cd como eje x
    clcd.plot(cdalfa,clalfa, linewidth=3)
    clcd.grid(True,linestyle='--')
    plt.title('CL vs CD',fontsize=16)
    plt.xlabel('CD',fontsize=12)
    plt.ylabel('CL',fontsize=12)
    
    fig.tight_layout() #evita que se superpongan los graficos
    
    
    plt.show() #muestra el grafico
		
    
#ondas calcula el segundo punto de la recta que representa la linea de mach, se entra con el punto inicial (tomado desde el origen) y el angulo de la onda, devuelve una lista con los puntos finales, aplicando la matriz de rotacion y suponiendo modulo de la recta = 5000 (para que sean infinitas).
def ondas(x,y,alfainf):
    xfinal=5000*math.cos(alfainf)+x
    yfinal=0+5000*math.sin(alfainf)+y
    
    puntosfinal=[xfinal,yfinal]
    
    return puntosfinal
    
    
	
#dibuja el perfil, las ondas, etc
def dibuja():
    global tita
    tita=math.atan((0.5*c*esp/100.)/(c/2)) #tita es el angulo entre el primer panel del extrados y la cuerda
    #delta=math.atan((0.5*c*esp/100.)/(c/2))+alfa   #angulo de quiebre del panel anterior, recordar que alfas positivos en el grafico son negativos en el programa, es decir, cuando aumenta el alfa (grafico) disminuye delta
    
    caly=200. #valores para afinal la posicion de los dibujos
    calx=50.
    intervalo_mach=2.*caly/5. #Separa las lineas que representan el flujo libre, en y. *2 para que el perfil este centrado y dividido 5 porque es la cantidad de lineas que hay
        
    delta=tita+float(alfa) #mismo delta de antes
    delta2=tita+abs(alfa) #delta mas grande encontrado en el perfil, por eso el absoluto de alfa
    
    
    canvas.delete(Tkinter.ALL) #borra lo dibujado para que no se superpongan
    
    
    ##usar un ofset gral y aplicarle la matriz o moverlos al origen y aplicar rotacion
    x=[0,0,0,0,0,0] #lista de puntos x de los paneles, para generar el poligono
    y=[0,0,0,0,0,0] #idem anterior pero en y
    #el centro del perfil coincide con el origen, se rota y se desplaza desde ahi
    x[0]=(-c/2.*math.cos(alfa))+(w_max/2.+calx) #se calculan los puntos y se les aplica la matriz de rotacion, los puntos se calculan en sentido horario partiendo del borde de ataque
    y[0]=(-c/2.*-math.sin(alfa)+0.)+(caly)
    
    x[1]=(0.+(-0.5*c*esp/100.)*math.sin(alfa))+((w_max/2.)+calx)
    y[1]=(-(0.5*c*esp/100.)*math.cos(alfa))+(caly)    
    
    x[2]=(c/2.*math.cos(alfa))+(w_max/2.+calx)
    y[2]=(0.+c/2.*-math.sin(alfa))+(caly)    
    
    x[3]=(0.+((0.5*c*esp/100.)*math.sin(alfa)))+(w_max/2.+calx)
    y[3]=(0.5*c*esp/100.)*math.cos(alfa)+(caly)
    
    
    x[4]=x[0] #hay puntos que son iguales, pero se necesitan en dos posiciones de la lista al momento de calcular las lineas de mach
    x[5]=x[2]
    y[4]=y[0]
    y[5]=y[2]
    
    
    
    points=[] #lista a llenar con los puntos del poligono (perfil)
    
    for i in range(4):
        points.append(x[i])
        points.append(y[i])
    
    
    ##Color del perfil
    
    if delta2>0.08729 and delta2<0.08729*2: #si algun quiebre supera los 5 grados (rad) se pinta de amarillo
        colorfill="yellow"
    elif delta2>=0.08729*2: #y si supera los 10 grados, de rojo
        colorfill="red"
    else:
        colorfill="black" #si esta dentro de los limites de teoria linealizada, se pinta de negro
    
    canvas.create_polygon(points,outline="black", fill=colorfill, width=2) #dibuja el perfil como un poligono
    
     
       
    
    ##Flujo libre, dibuja las lineas del flujo libre que representan la velocidad que tiene, como el mach es chico, este se multiplica por 30, y se le suman 50 para separarlas del borde
    canvas.create_line(50, intervalo_mach*1,    M*30+50,  intervalo_mach*1,width=2, arrow=Tkinter.LAST)
    canvas.create_line(50, intervalo_mach*2,    M*30+50,  intervalo_mach*2,width=2, arrow=Tkinter.LAST)
    canvas.create_line(50, intervalo_mach*3,    M*30+50,  intervalo_mach*3,width=2, arrow=Tkinter.LAST)
    canvas.create_line(50, intervalo_mach*4,    M*30+50,  intervalo_mach*4,width=2, arrow=Tkinter.LAST)
    
    
    ##Onda de choque y expansion
    clin=["a","a","a","a","a","a"] #lista llena de strings que va a representar el color de las ondas
    lin=[0,0,0,0,0,0] #lista llena que va a representar el espesor de las ondas
    
    #orden de las ondas:
    #0 1 2
    #3 4 5
    
    #hay que determinar si son de compresion (rojas) o expansion (azules)
    if delta>0: #significa compresion en el panel 1 (primero de extrados)
        ##lin[0]=5
        lin[0]=(delta*57.295/7) #espesor proporcional al quiebre ->proporcional a la intensidad de la onda
        clin[0]="red"
    else: #si no es compresion, es expansion
        lin[0]=1
        clin[0]="blue"
		
    if tita<alfa: #hace lo mismo que el anterior pero para la onda del borde de fuga, la onda central es siempre de expansion
        lin[1]=1
        clin[1]="blue"
        lin[2]=1
        clin[2]="blue"
    else:
        lin[1]=1
        clin[1]="blue"
        ##lin[2]=5
        lin[2]=((tita-alfa)*57.295/7)
        clin[2]="red"
        
        
    #ahora intrados
    
    if -tita+alfa<0:
        ##lin[4]=5
        lin[4]=((tita-alfa)*57.295/7)
        clin[4]="red"
    else:
        lin[4]=1
        clin[4]="blue"
		
    if -tita>alfa:
        lin[3]=1
        clin[3]="blue"
        lin[5]=1
        clin[5]="blue"
    else:
        lin[3]=1
        clin[3]="blue"
        ##lin[5]=5
        lin[5]=(delta*57.295/7)
        clin[5]="red"
    
    
    
	##calculo de las ondas
	# 0 1 2 
	# 3 4 5
    beta[0]=(-math.asin(1/M)-tita-alfa) #angulos de las ondas, recordar que el sentido de alfa es opuesto al del grafico
    beta[1]=(-math.asin(1/M)-tita-alfa)
    beta[2]=(-math.asin(1/M)-tita-alfa)
    beta[3]=(math.asin(1/M)+tita-alfa)
    beta[4]=(math.asin(1/M)+tita-alfa)
    beta[5]=(math.asin(1/M)+tita-alfa)
	
    puntotempx=[] #listas de puntos con las coordenadas finales de las ondas, para graficarlas
    puntotempy=[]
    
    i=0
    while i<6:
        puntotempx.append(ondas(x[i],y[i],beta[i])[0]) #devuelve el valor [0] de ondas, es decir, el valor en x final de la onda que le pedi que calcule
        puntotempy.append(ondas(x[i],y[i],beta[i])[1]) #idem en y
        i=i+1
    
    i=0
    while i<6:    #grafica todas las ondas
       canvas.create_line(x[i],y[i], puntotempx[i],puntotempy[i],width=lin[i],fill=clin[i])
       i=i+1
    
    
   
    
    ##Calculo del Cl y Cd a 10000m y cuerda unitaria
    cl=(4*-alfa)/((M**2-1)**0.5) #formulas del shappiro, se usan el alfa y mach seleccionados en los sliders
    L=0.5*1.4*26434.75*M**2*cl
    cd=(4*alfa**2)/((M**2-1)**0.5)+(4*tita**2)/((M**2-1)**0.5)
    D=0.5*1.4*26434.75*M**2*cd
    
    #Coordenadas de calibracion del texto que muestra el cl, cd, L y D
    xtexto=200
    ytexto=500
    
    #se crea un poligono que borra (sobreescribe) las lineas de mach  en el lugar donde va el texto
    puntospoligono=[xtexto-500,ytexto-100,xtexto+1000,ytexto-100,xtexto+800,ytexto+150,xtexto-500,ytexto+150]
    canvas.create_polygon(puntospoligono,outline="grey",fill="white", width=1)    
    
    #se escribe el texto
    canvas.create_text(xtexto+200,ytexto-70,font="Verdana 20",text=("10000m de altitud"))
    canvas.create_text(xtexto,ytexto,font="Verdana 20 italic",text="Cl = "+str(round(cl,6)))
    canvas.create_text(xtexto,ytexto+50,font="Verdana 20 italic",text="L ="+ str(round(L,2))+" N/m**2")
    canvas.create_text(xtexto+400,ytexto,font="Verdana 20 italic",text="Cd ="+str(round(cd,6)))
    canvas.create_text(xtexto+400,ytexto+50,font="Verdana 20 italic",text="D ="+ str(round(D,2))+" N/m**2")
     
    
    
    
    
    canvas.update() #actualiza el dibujo
    
    return




#def boton():
#    dibuja()
#    return
    
def boton2(): #crea los graficos con el tita (espesor relativo) seleccionado
    calcular(tita)
    return

#todos los sliders incluyen dibuja() para que la representacion sea instantanea
def alfa(alfa1):
    global alfa
    alfa=float(alfa1)/-57.295 #valor del slider en radianes, parece que el menos de antes era culpa de esto, funciona y si se puede no cambiar mejor
    dibuja()
    return 
    
def espesor(esp1): #lee el espesor relativo
    global esp
    esp=float(esp1)
    dibuja()
    return
    
def mach(M1): #lee el numero de mach
    global M
    M=float(M1)
    dibuja()
    return




#crea la venana del dibujo
root = Tkinter.Tk()
root.title("Perfiles Supersonicos")
canvas = Tkinter.Canvas(root,width=w_max, height=h_max+200,bg='white')
canvas.grid(column=0, row=0,columnspan=2,rowspan=3) 


#se crean los botones
#b = Tkinter.Button(root, text ="Ok", command = boton)
#b.grid(row=4, column=6)
h = Tkinter.Button(root, text ="Calcular", command = boton2)
h.grid(row=4, column=6)


#se crean los sliders
lab1=Tkinter.Label(root,text="ANGULO DE ATAQUE")
lab1.grid(row=0,column=4,sticky=Tkinter.E)
root_x=Tkinter.Scale(root,orient="horizontal",command=alfa,from_=-15.0,to=15.0,resolution=1.0)
root_x.grid(row=0,column=5)

lab2=Tkinter.Label(root,text="ESPESOR RELATIVO")
lab2.grid(row=1,column=4,sticky=Tkinter.E)
root_y=Tkinter.Scale(root,orient="horizontal",command=espesor,from_=1.0,to=15.0,resolution=1.0)
root_y.grid(row=1,column=5)

lab3=Tkinter.Label(root,text="NUMERO DE MACH")
lab3.grid(row=2,column=4,sticky=Tkinter.E)
root_g=Tkinter.Scale(root,orient="horizontal",command=mach,from_=1.2,to=4.0,resolution=0.1)
root_g.grid(row=2,column=5)

#variables inicializadas de nuevo, mania del codigo
alfa=0.
esp=1.
M=1.2
dibuja() #dibuja el primero, antes de tocar nada
root.mainloop() #mantiene el programa dentro del loop, esperando a hacer algo
